// Fichier main.c
/////////////////////////////////////////////////////////

/****************************************************************
 *                                                    			*
 * .Date de cration : 11/01/08                         		*
 * .Auteur : Guibert David                              		*       
 *           Cadic Matthieu										*
 *           Luzeau Yohan                                		*
 * .Descriptions :                                      		*
 *      - Fichier principal pour la gestion du rouleau droit   	*
 *                                                      		*
 *                                                      		*
 *                                                      		*
 * .Remarques :                                         		*
 *      - Gestion des ordre reus par interruption plutt		*
 *		  que par polling								   		*
 *      - Adresse de la carte droite 0x4E          				*
 * 		- Transmission IC sur interruption (priorit haute)	*
 * 		- Timer sur interruption (priorit basse)               *
 *		- La led rouge marque l'interruption timer				*
 *		- La led ambre marque une transmission IC				*
 *																*
 ****************************************************************/




// ------------ Directives de compilation
#pragma config WDT = OFF	// Pas de  watchdog
#pragma config OSC = HS		// Quartz HS pour un quartz 20MHz


// --------------------------------------------------------------------------
// Enttes ncessaires
#include <p18f252.h>		// Pour les registres du PIC18F252
#include <delays.h>    		// Pour les temporisations
#include <timers.h>			// Pour la base de temps de 10ms
#include <adc.h>			// Bibliothque des convertisseurs Analogiques Numriques
#include <stdio.h>

#include "Moteur.h"			// Librairie de gestion du moteur
#include "I2C_Slave.h"		// Librairie pour l'I2C
#include "led.h"			// Librairie pour les leds
#include "codeur.h"			// Librairie pour le codeur
#include "serial.h"

// --------------------------------------------------------------------------
// Constantes

//#define		K			2.202609867
//#define		Alpha		0.93299

#define		K			0.7
#define		Alpha		0.93299

#define 	Kp			2.20260987			// Kp= (1-a)/b
#define 	Ki			0.147596887			// Ki= Kp-a*Kp




#define 	T_TIMER3 	40535			   	// 5 ms avant dbordement du Timer
											// Priode du quartz : 1/20e6 = 50ns
											// Priode  l'entre du prescaler : 50ns.4 = 200ns
											// Priode  la sortie du prescaler : 200ns.1 = 200ns
											// Nombre de priodes avant dbordement du timer : 10000 incrments
											// Priode entre deux interruptions : 25000*200ns = 5ms
											// Priode de 1ms -> 1KHz
											// 60536 pour 1ms
											// 55536 pour 2ms
											// 15535 pour 10ms

// --------------------------------------------------------------------------
// Variables partage
volatile int 			Speed_Shared=0;					// Vitesse mesure au codeurs
volatile int 			Consigne_Shared=0;				// Consigne dsire
volatile unsigned char 	CompteurLedAmbre_Shared=0;		// Compteur pour allumer la led ambre sur un dialogue IC



// --------------------------------------------------------------------------

// Prototype de la routine de service d'interuptions de priorit haute
void Interrupt_High_Priority(void);					// Utilise pour l'IC


// Prototype de la routine de service d'interuptions de priorit basse
void Interrupt_Low_Priority(void);					// Utilise pour le timer




/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                            Fonction Principale                          //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

void main (void)
{
	// Configure le micro	
	
	OpenADC (ADC_0ANA_0REF,ADC_INT_OFF);	// Desactive les convertisseurs Analogiques/Numriques
	TRISBbits.TRISB1=1;
	TRISBbits.TRISB0=1;
	Moteur_Init();							// Initialise la commande du moteur
 	I2C_Init ();							// Initialise l'I2C
 	Led_Init();								// Configure les ports pour les leds
 	Codeur_Init();
	Serial_Init();
	
	
	
	// Configuration du timer 3 pour la base de temps
	
	OpenTimer3(	TIMER_INT_ON &				// Autorise les interruption sur le timer 3
				T3_16BIT_RW &				// Compteur 16 bits
				T3_SOURCE_INT &				// Source interne (Quartz Fosc/4)
				T3_PS_1_1 &					// Prescaler 1/1
				T3_SYNC_EXT_OFF);			// Pas de synchronisation externe

	WriteTimer3(T_TIMER3);					// Charge le compteur du timer 3 avec la valeur de T_Timer3
	PIR2bits.TMR3IF=0;						// Timer3 Interrupt Flag = 0 Baisse le flag du timer 3
	IPR2bits.TMR3IP=0;						// Timer 3 dclench sur priorit basse
	RCONbits.IPEN=1;						// Autorise les niveau de priorit sur les interuptions
	
	TRISBbits.TRISB1=1;
	TRISBbits.TRISB0=1;
	
	
	// Autorise les interruptions	
	INTCONbits.PEIE=1;						// Autorise les interruptions de basse priorit
	INTCONbits.GIE=1;						// Autorise les interruptions de haute priorit
		
	while(1);								// Rien a faire, tout est gr dans la routine d'interruption
}






/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                            Interruptions                                // 
//                                                                         // 
/////////////////////////////////////////////////////////////////////////////



//----------------------------------------------------------------------------
// Vecteur d'interruptions de priorit haute (utilis pour l'IC)
#pragma code highVector=0x08						// Vecteur d'interruption haut
void atInterruptHigh(void)
{
	_asm goto Interrupt_High_Priority _endasm		// On saute  la fonction MyHighInterrupt
}
#pragma code 										// Retour  la zone de code




//----------------------------------------------------------------------------
// Vecteur d'interruptions de priorit basse (utilis pour le Timer)
#pragma code lowVector=0x18							// Vecteur d'interruption de priorit basse
void atInterruptLow(void)
{
	_asm goto Interrupt_Low_Priority _endasm		// On saute  la fonction MyHighInterrupt
}
#pragma code 										// Retour  la zone de code




//----------------------------------------------------------------------------
// Routine de service de l'interruption haute priorite
#pragma interrupt Interrupt_High_Priority
void Interrupt_High_Priority(void)
{
	
	// ATENTION, PENSER A VERROUILLER SPEED +128 -128
	
	if (PIR1bits.SSPIF) 									// C'est le MSSP qui a lev l'interruption, c'est pour l'IC
	{
		CompteurLedAmbre_Shared=50;							// Compteur (qui allume la led pendant 50ms)		
		PIR1bits.SSPIF = 0; 								// On rautorise l'interruption		
		if (Speed_Shared> 127) Speed_Shared= 127;			// Borne la vitesse entre [-128;+127]
		if (Speed_Shared<-128) Speed_Shared=-128;			// Pour l'envoie par IC
		Consigne_Shared=InterruptI2C((char)Speed_Shared);	// On appelle la routine I2C pour traiter la requete				
	}	
}




//----------------------------------------------------------------------------
// Routine de service de l'interruption haute priorite
#pragma interrupt Interrupt_Low_Priority
void Interrupt_Low_Priority(void)
{
	int Consigne=63;										// Consigne  appliquer sur le moteur
	int Speed;											// Vitesse mesure au codeur
	float Commande=0;
	static float CommandeOld=0;
	float Error=0;
	static float ErrorOld=0;
		
	if(PIR2bits.TMR3IF)									// Vrifie qu'il s'agit bien de l'interruption du Timer 3
	{									
		Led_Rouge(1);									// Pour marquer le dbut de l'interruption
		
		
		WriteTimer3(T_TIMER3);							// Recharge le compteur du Timer 3		
		PIR2bits.TMR3IF=0;								// Rautotise l'interruption


		Speed=Codeur_Vitesse();							// Lecture de la vitesse du moteur
				
		printf ("%d %d %d\n",Consigne,Speed,(int)CommandeOld);
		
		Error=(float)(Consigne-Speed);
		//Commande=K*Alpha*Error-K*ErrorOld+CommandeOld;

		Commande=CommandeOld+(Ki-Kp)*ErrorOld+Kp*Error;
		
		CommandeOld=Commande;
		ErrorOld=Error;
		//CommandeMoteur (Consigne*8);   					// On commande la vitesse du moteur avec la valeur transmise				

		if (Commande>1023) Commande=1023;
		if (Commande<-1023) Commande=-1023;

		CommandeMoteur ((int)Commande);	
	
		
		Led_Rouge(0);									// Pour marquer la fin de l'interruption
	}
}
	